//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Depth.h"
#include "StringHelp.h"
#include <shellapi.h>
#include <math.h>
#include <IniFiles.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDepthCutForm *DepthCutForm;
//---------------------------------------------------------------------------
__fastcall TDepthCutForm::TDepthCutForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TDepthCutForm::FormCreate(TObject *Sender)
{
    //Open Persistant Data & initialise variables
    TIniFile *IniFile = new TIniFile(ExtractFileDir(Application->ExeName) + "\\Data.ini");

    Edit3->Text = IniFile->ReadString("DepthMill","LeftX","0.0");
    Edit4->Text = IniFile->ReadString("DepthMill","WrapAngle","360");
    Edit5->Text = IniFile->ReadString("DepthMill","ImageLength","100.0");
    Edit6->Text = IniFile->ReadString("DepthMill","WorkSurface","7.5");
    Edit7->Text = IniFile->ReadString("DepthMill","DepthofCut","1.0");
    Edit8->Text = IniFile->ReadString("DepthMill","Feedrate","200");
    Edit9->Text = IniFile->ReadString("DepthMill","Clearance","10");
    Edit10->Text = IniFile->ReadString("GCode","Header","");
    Edit11->Text = IniFile->ReadString("GCode","Footer","M2");
    OpenDialog1->InitialDir = IniFile->ReadString("DepthMill","InitialDir",ExtractFileDir(Application->ExeName));
    //ComboBox1->ItemIndex = IniFile->ReadInteger("DepthMill","Axis",1);
    ComboBox2->ItemIndex = IniFile->ReadInteger("DepthMill","RotaryAxis",0);

    delete IniFile;

    Picture = new Graphics::TBitmap();
    ConversionPossible = false;


}
//---------------------------------------------------------------------------
void __fastcall TDepthCutForm::FormDestroy(TObject *Sender)
{
    //Save Persistant Data
    TIniFile *IniFile = new TIniFile(ExtractFileDir(Application->ExeName) + "\\Data.ini");

    IniFile->WriteString("DepthMill","LeftX",Edit3->Text);
    IniFile->WriteString("DepthMill","WrapAngle",Edit4->Text);
    IniFile->WriteString("DepthMill","ImageLength",Edit5->Text);
    IniFile->WriteString("DepthMill","WorkSurface",Edit6->Text);
    IniFile->WriteString("DepthMill","DepthofCut",Edit7->Text);
    IniFile->WriteString("DepthMill","Feedrate",Edit8->Text);
    IniFile->WriteString("DepthMill","Clearance",Edit9->Text);
    IniFile->WriteString("GCode","Header",Edit10->Text);
    IniFile->WriteString("GCode","Footer",Edit11->Text);
    IniFile->WriteString("DepthMill","InitialDir",OpenDialog1->InitialDir);
    //IniFile->WriteInteger("DepthMill","Axis",ComboBox1->ItemIndex);
    IniFile->WriteInteger("DepthMill","RotaryAxis",ComboBox2->ItemIndex);

    delete IniFile;
    delete Picture;

}
//---------------------------------------------------------------------------

void __fastcall TDepthCutForm::OriginalBMPButtonClick(TObject *Sender)
{
    if (OpenDialog1->Execute()) {
        Edit1->OnChange = NULL;
        Edit1->Text = OpenDialog1->FileName;
        OpenDialog1->InitialDir = ExtractFilePath(Edit1->Text);
        Edit2->Text = ChangeFileExt(Edit1->Text, " Depth Mill.dxf");
        SaveDialog1->InitialDir = ExtractFilePath(Edit1->Text);
        try {
            Picture->LoadFromFile(Edit1->Text);
            Label3->Caption = "Picture loaded and is = " + AnsiString(Picture->Width) + "W x " + AnsiString(Picture->Height) + "H Pixels";
            ConversionPossible = true;
            SetButtons(true);
            }
        catch (...) {
            Label3->Caption = "Picture did not load";
            ConversionPossible = false;
            SetButtons(false);
            }
        Edit1->OnChange = Edit1Change;
        }

}
//---------------------------------------------------------------------------
void __fastcall TDepthCutForm::NewDXFButtonClick(TObject *Sender)
{
    if (SaveDialog1->Execute()) {
        Edit2->Text = SaveDialog1->FileName;
        SaveDialog1->InitialDir = ExtractFilePath(Edit2->Text);
        }

    
}
//---------------------------------------------------------------------------
void __fastcall TDepthCutForm::CreateDXFButtonClick(TObject *Sender)
{
    if (!Edit1->Text.Length() || !Edit2->Text.Length()) {
        Application->MessageBox("Need two file names\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (Edit1->Text.LowerCase() == Edit2->Text.LowerCase()) {
        Application->MessageBox("Both file names the same\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (!Edit4->Text.Length()) {
        Application->MessageBox("Please enter a maximum value for\nthe angle subtended by the image\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit5->Text,0.0) == 0.0) {
        Application->MessageBox("The Image has zero length along the X Axis.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit6->Text,0.0) == 0.0) {
        Application->MessageBox("The nominal work surface is at the rotary Axis centre.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit7->Text,0.0) == 0.0) {
        Application->MessageBox("The depth of cut is zero, so no machining will occur.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (FileExists(Edit1->Text) && !FileExists(Edit2->Text)) {

        //Create Depth Array
        DynamicArray< DynamicArray < double > > Depths;
        Depths.Length = Picture->Width;
        for (int i = 0; i < Depths.Length; i++) {
            Depths[i].Length = Picture->Height;
            }

        try {
            SetInterface(false);

            //obtain values from input data
            double XOffset = ToDoubleDef(Edit3->Text,0.0);
            double TotalAngle = ToDoubleDef(Edit4->Text,360.0);
            //angle per pixel in degrees
            double angleperpixel = (M_PI * TotalAngle)/(180.0 * (double) Picture->Height);
            double TotalLength = ToDoubleDef(Edit5->Text,100.0);
            double lengthperpixel = TotalLength/((double)(Picture->Width-1));
            double NominalWorkSurface = ToDoubleDef(Edit6->Text,7.5);
            double DepthofCut = ToDoubleDef(Edit7->Text,1.0);
            double depthfactor;
            double angle;
            double radius;

            //go through pixels & find the maximum brightness of the pixels in the picture
            int maxbrightness = 0;
            int localbrightness;
            TColor pixelcolour;
            int red;
            int green;
            int blue;

            //first loop to find out maxbrightness values
            Label3->Caption = "Checking max brightness values";
            Refresh();

            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    if (localbrightness > maxbrightness) maxbrightness = localbrightness;
                    }
                }
            //right, max brightness now holds the peak brightness
            depthfactor = DepthofCut/((double)maxbrightness);

            //Assign depths
            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    Depths[i][j] = ((double) localbrightness) * depthfactor;
                    }
                }

            //right, all the individual depths are stored in the array, now to write a DXF file
            Label3->Caption = "Creating DXF file";
            Refresh();
            TStringList *DXFOutputFile = new TStringList();
            //Add Header
            DXFOutputFile->Add("999");
            DXFOutputFile->Add("DXF File Produced by the Chestnut Pens Depth Mill Program");
            DXFOutputFile->Add("999");
            DXFOutputFile->Add("Program Copyright http://www.chestnutpens.co.uk");

            //Table
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("SECTION");
            DXFOutputFile->Add(" 2");
            DXFOutputFile->Add("TABLES");

            //Line type table
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("TABLE");
            DXFOutputFile->Add(" 2");
            DXFOutputFile->Add("LTYPE");
            DXFOutputFile->Add("70");
            DXFOutputFile->Add("1");
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("LTYPE");
            DXFOutputFile->Add(" 2");
            DXFOutputFile->Add("CONTINUOUS");
            DXFOutputFile->Add("70");
            DXFOutputFile->Add("64");
            DXFOutputFile->Add(" 3");
            DXFOutputFile->Add("Solid Line");
            DXFOutputFile->Add("72");
            DXFOutputFile->Add("65");
            DXFOutputFile->Add("73");
            DXFOutputFile->Add("0");
            DXFOutputFile->Add("40");
            DXFOutputFile->Add("0.000000");
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("ENDTAB");

            //Layer Table
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("TABLE");
            DXFOutputFile->Add(" 2");
            DXFOutputFile->Add("LAYER");
            DXFOutputFile->Add("70");
            DXFOutputFile->Add("6");
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("LAYER");
            DXFOutputFile->Add(" 2");
            DXFOutputFile->Add("Chestnut");
            DXFOutputFile->Add("70");
            DXFOutputFile->Add("64");
            DXFOutputFile->Add("62");
            DXFOutputFile->Add("5");
            DXFOutputFile->Add(" 6");
            DXFOutputFile->Add("CONTINUOUS");
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("ENDTAB");

            //Style table
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("TABLE");
            DXFOutputFile->Add(" 2");
            DXFOutputFile->Add("STYLE");
            DXFOutputFile->Add("70");
            DXFOutputFile->Add("0");
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("ENDTAB");

            //end tables
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("ENDSEC");

            //Print the Entities
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("SECTION");
            DXFOutputFile->Add(" 2");
            DXFOutputFile->Add("ENTITIES");

            //Now all the 3D Surfaces
            for (int i = 0; i < Depths.Length - 1; i++) {
                for (int j = 0; j < Depths[i].Length - 1; j++) {
                    DXFOutputFile->Add(" 0");
                    DXFOutputFile->Add("3DFACE");
                    DXFOutputFile->Add(" 8");
                    DXFOutputFile->Add("Chestnut");
                    DXFOutputFile->Add("62");
                    DXFOutputFile->Add("83");
                    //corner 1 of surface
                    radius = NominalWorkSurface - Depths[i][j];
                    angle = ((double)(j)) * angleperpixel;
                    DXFOutputFile->Add("10");
                    DXFOutputFile->Add(AnsiString(XOffset + lengthperpixel * (double)(i)));
                    DXFOutputFile->Add("20");
                    DXFOutputFile->Add(AnsiString(radius * cos(angle) /* * -1.0 */));
                    DXFOutputFile->Add("30");
                    DXFOutputFile->Add(AnsiString(radius * sin(angle)));
                    //corner 2 of surface
                    radius = NominalWorkSurface - Depths[i + 1][j];
                    DXFOutputFile->Add("11");
                    DXFOutputFile->Add(AnsiString(XOffset + lengthperpixel * (double)(i+1)));
                    DXFOutputFile->Add("21");
                    DXFOutputFile->Add(AnsiString(radius * cos(angle) /* * -1.0 */));
                    DXFOutputFile->Add("31");
                    DXFOutputFile->Add(AnsiString(radius * sin(angle)));
                    //corner 3 of surface
                    radius = NominalWorkSurface - Depths[i+1][j+1];
                    angle = ((double)(j + 1)) * angleperpixel;
                    DXFOutputFile->Add("12");
                    DXFOutputFile->Add(AnsiString(XOffset + lengthperpixel * (double)(i+1)));
                    DXFOutputFile->Add("22");
                    DXFOutputFile->Add(AnsiString(radius * cos(angle) /* * -1.0*/));
                    DXFOutputFile->Add("32");
                    DXFOutputFile->Add(AnsiString(radius * sin(angle)));
                    //corner 4 of surface
                    radius = NominalWorkSurface - Depths[i][j+1];
                    DXFOutputFile->Add("13");
                    DXFOutputFile->Add(AnsiString(XOffset + lengthperpixel * (double)(i)));
                    DXFOutputFile->Add("23");
                    DXFOutputFile->Add(AnsiString(radius * cos(angle) /* * -1.0*/));
                    DXFOutputFile->Add("33");
                    DXFOutputFile->Add(AnsiString(radius * sin(angle)));
                    }
                }
            if (TotalAngle == 360.0) {
                for (int i = 0; i < Depths.Length - 1; i++) {
                    DXFOutputFile->Add(" 0");
                    DXFOutputFile->Add("3DFACE");
                    DXFOutputFile->Add(" 8");
                    DXFOutputFile->Add("Chestnut");
                    DXFOutputFile->Add("62");
                    DXFOutputFile->Add("83");
                    //corner 1 of surface
                    radius = NominalWorkSurface - Depths[i][Depths[i].Length - 1];
                    angle = ((double)(Depths[i].Length - 1)) * angleperpixel;
                    DXFOutputFile->Add("10");
                    DXFOutputFile->Add(AnsiString(XOffset + lengthperpixel * (double)(i)));
                    DXFOutputFile->Add("20");
                    DXFOutputFile->Add(AnsiString(radius * cos(angle)/* * -1.0*/));
                    DXFOutputFile->Add("30");
                    DXFOutputFile->Add(AnsiString(radius * sin(angle)));
                    //corner 2 of surface
                    radius = NominalWorkSurface - Depths[i + 1][Depths[i].Length - 1];
                    DXFOutputFile->Add("11");
                    DXFOutputFile->Add(AnsiString(XOffset + lengthperpixel * (double)(i+1)));
                    DXFOutputFile->Add("21");
                    DXFOutputFile->Add(AnsiString(radius * cos(angle) /* * -1.0*/));
                    DXFOutputFile->Add("31");
                    DXFOutputFile->Add(AnsiString(radius * sin(angle)));
                    //corner 3 of surface
                    radius = NominalWorkSurface - Depths[i+1][0];
                    angle = ((double)(0)) * angleperpixel;
                    DXFOutputFile->Add("12");
                    DXFOutputFile->Add(AnsiString(XOffset + lengthperpixel * (double)(i+1)));
                    DXFOutputFile->Add("22");
                    DXFOutputFile->Add(AnsiString(radius * cos(angle) /* * -1.0*/));
                    DXFOutputFile->Add("32");
                    DXFOutputFile->Add(AnsiString(radius * sin(angle)));
                    //corner 4 of surface
                    radius = NominalWorkSurface - Depths[i][0];
                    DXFOutputFile->Add("13");
                    DXFOutputFile->Add(AnsiString(XOffset + lengthperpixel * (double)(i)));
                    DXFOutputFile->Add("23");
                    DXFOutputFile->Add(AnsiString(radius * cos(angle) /* * -1.0*/));
                    DXFOutputFile->Add("33");
                    DXFOutputFile->Add(AnsiString(radius * sin(angle)));
                    }
                }


            //End
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("ENDSEC");
            DXFOutputFile->Add(" 0");
            DXFOutputFile->Add("EOF");

            Label3->Caption = "Writing DXF file";
            Refresh();

            DXFOutputFile->SaveToFile(Edit2->Text);
            delete DXFOutputFile;

            Refresh();
            }  //end of try  statement

        __finally {
            SetInterface(true);
            //get rid of depth array
            for (int i = 0; i < Depths.Length; i++) {
                Depths[i].Length = 0;
                }
            Depths.Length = 0;
            //end
            Label3->Caption = "Complete";
            Refresh();
            }

        } else {
        if (FileExists(Edit1->Text)) {
            Application->MessageBox("An existing DXF file will get overwritten.\n\nPlease delete file or change converted file name","Warning", MB_OK | MB_ICONERROR);
            } else {
            Application->MessageBox("The input file does not exist.\n\nPlease check","Warning", MB_OK | MB_ICONERROR);
            }
        Label3->Caption = "Conversion Aborted";
        }

}
//---------------------------------------------------------------------------
void __fastcall TDepthCutForm::HelpButtonClick(TObject *Sender)
{
    //Application->MessageBox("This program takes a G Code File, and wraps\nthe code around the X or Y axis.\n\nTo operate, select 2 file names, max segment & degree values for the\narcs to be converted to lines, then fill in the pertinant axis data\n and press the\n'Wrap' button.\n","Help on G Code Wrapper Program",MB_OK | MB_ICONINFORMATION);
    AnsiString Address = ExtractFilePath(Application->ExeName) + "\\Depth Mill Help.pdf";
    ShellExecute(Application->Handle,NULL,Address.c_str(),NULL,NULL,0);
    Address.~AnsiString();


}
//---------------------------------------------------------------------------
void __fastcall TDepthCutForm::CloseButtonClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------

void __fastcall TDepthCutForm::CreateSTLButtonClick(TObject *Sender)
{
    //similarly to the DXF file creation above, this creates an STL file
    if (!Edit1->Text.Length() || !Edit2->Text.Length()) {
        Application->MessageBox("Need two file names\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (Edit1->Text.LowerCase() == ChangeFileExt(Edit2->Text.LowerCase(),".stl")) {
        Application->MessageBox("Both file names the same\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (!Edit4->Text.Length()) {
        Application->MessageBox("Please enter a maximum value for\nthe angle subtended by the image\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit5->Text,0.0) == 0.0) {
        Application->MessageBox("The Image has zero length along the X Axis.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit6->Text,0.0) == 0.0) {
        Application->MessageBox("The nominal work surface is at the rotary Axis centre.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit7->Text,0.0) == 0.0) {
        Application->MessageBox("The depth of cut is zero, so no machining will occur.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (FileExists(Edit1->Text) && !FileExists(ChangeFileExt(Edit2->Text,".stl"))) {

        //Create Depth Array
        DynamicArray< DynamicArray < double > > Depths;
        Depths.Length = Picture->Width;
        for (int i = 0; i < Depths.Length; i++) {
            Depths[i].Length = Picture->Height + 1;
            }

        try {
            SetInterface(false);

            //obtain values from input data
            double XOffset = ToDoubleDef(Edit3->Text,0.0);
            double TotalAngle = ToDoubleDef(Edit4->Text,360.0);
            //angle per pixel in degrees
            double angleperpixel = (M_PI * TotalAngle)/(180.0 * (double) Picture->Height);
            double TotalLength = ToDoubleDef(Edit5->Text,100.0);
            double lengthperpixel = TotalLength/((double) (Picture->Width - 1));
            double NominalWorkSurface = ToDoubleDef(Edit6->Text,7.5);
            double DepthofCut = ToDoubleDef(Edit7->Text,1.0);
            double depthfactor;
            double angle;
            double radius;
            DynamicArray <nodalposition> FacetCorners;
            FacetCorners.Length = 4;
            nodalposition FacetNormal;
            nodalposition v1, v2;


            //go through pixels & find the maximum brightness of the pixels in the picture
            int maxbrightness = 0;
            int localbrightness;
            TColor pixelcolour;
            int red;
            int green;
            int blue;

            //first loop to find out maxbrightness values
            Label3->Caption = "Checking max brightness values";
            Refresh();

            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    if (localbrightness > maxbrightness) maxbrightness = localbrightness;
                    }
                }
            //right, max brightness now holds the peak brightness
            depthfactor = DepthofCut/((double)maxbrightness);

            //Assign depths
            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    Depths[i][j] = ((double) localbrightness) * depthfactor;
                    }
                }
            //now all the depths are assigned, except the last row, which is a copy of the first
            //this simplifies the closure process, which must occur on STL files
            for (int i = 0; i < Picture->Width; i++) {
                Depths[i][Depths[i].Length - 1] = Depths[i][0];
                }


            //right, all the individual depths are stored in the array, now to write a DXF file
            Label3->Caption = "Creating STL file";
            Refresh();
            TStringList *STLOutputFile = new TStringList();
            //Add Header
            STLOutputFile->Add("solid " + ExtractFileName(ChangeFileExt(Edit2->Text,".stl")));
            //Now all the 3D Surfaces
            for (int i = 0; i < Depths.Length - 1; i++) {
                for (int j = 0; j < Depths[i].Length - 1; j++) {
                    //Facet corners
                    //corner 1 of surface
                    radius = NominalWorkSurface - Depths[i][j];
                    angle = ((double)(j)) * angleperpixel;
                    FacetCorners[0].x = XOffset + lengthperpixel * (double)(i);
                    FacetCorners[0].y = radius * cos(angle);
                    FacetCorners[0].z = radius * sin(angle);
                    //corner 2 of surface
                    radius = NominalWorkSurface - Depths[i + 1][j];
                    FacetCorners[1].x = XOffset + lengthperpixel * (double)(i+1);
                    FacetCorners[1].y = radius * cos(angle);
                    FacetCorners[1].z = radius * sin(angle);
                    //corner 3 of surface
                    radius = NominalWorkSurface - Depths[i+1][j+1];
                    angle = ((double)(j + 1)) * angleperpixel;
                    FacetCorners[2].x = XOffset + lengthperpixel * (double)(i+1);
                    FacetCorners[2].y = radius * cos(angle);
                    FacetCorners[2].z = radius * sin(angle);
                    //corner 4 of surface
                    radius = NominalWorkSurface - Depths[i][j+1];
                    FacetCorners[3].x = XOffset + lengthperpixel * (double)(i);
                    FacetCorners[3].y = radius * cos(angle);
                    FacetCorners[3].z = radius * sin(angle);

                    //Now to create normal & facet 1
                    v1 = SubtractVector(FacetCorners[2],FacetCorners[0]);
                    v2 = SubtractVector(FacetCorners[1],FacetCorners[0]);
                    FacetNormal = UnitVector(VectorProduct(v1,v2));
                    STLOutputFile->Add("    facet normal "  + FloatToStrF(FacetNormal.x,ffFixed,10,10) + " " + FloatToStrF(FacetNormal.y,ffFixed,10,10) + " " + FloatToStrF(FacetNormal.z,ffFixed,10,10));
                    //STLOutputFile->Add("    facet");
                    STLOutputFile->Add("      outer loop");
                    STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));
                    STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[2].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[2].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[2].z,ffFixed,10,10));
                    STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[1].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[1].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[1].z,ffFixed,10,10));
                    STLOutputFile->Add("      endloop");
                    STLOutputFile->Add("    endfacet");

                    //Now to create normal & facet 2
                    v1 = SubtractVector(FacetCorners[3],FacetCorners[0]);
                    v2 = SubtractVector(FacetCorners[2],FacetCorners[0]);
                    FacetNormal = UnitVector(VectorProduct(v1,v2));
                    STLOutputFile->Add("    facet normal "  + FloatToStrF(FacetNormal.x,ffFixed,10,10) + " " + FloatToStrF(FacetNormal.y,ffFixed,10,10) + " " + FloatToStrF(FacetNormal.z,ffFixed,10,10));
                    //STLOutputFile->Add("    facet");
                    STLOutputFile->Add("      outer loop");
                    STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));
                    STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[3].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[3].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[3].z,ffFixed,10,10));
                    STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[2].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[2].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[2].z,ffFixed,10,10));
                    STLOutputFile->Add("      endloop");
                    STLOutputFile->Add("    endfacet");
                    }
                }

            //Close Ends of tube
            for (int i = 0; i < Depths[0].Length - 1; i++) {
                //low X end
                radius = NominalWorkSurface - Depths[0][i + 1];
                angle = ((double)(i + 1)) * angleperpixel;
                FacetCorners[0].x = XOffset;
                FacetCorners[0].y = radius * cos(angle);
                FacetCorners[0].z = radius * sin(angle);
                //corner 2 of surface
                radius = NominalWorkSurface - Depths[0][i];
                angle = ((double)(i)) * angleperpixel;
                FacetCorners[1].x = XOffset;
                FacetCorners[1].y = radius * cos(angle);
                FacetCorners[1].z = radius * sin(angle);
                //corner 3 of surface
                FacetCorners[2].x = XOffset;
                FacetCorners[2].y = 0.0;
                FacetCorners[2].z = 0.0;
                //Now to create normal & facet
                STLOutputFile->Add("    facet normal -1.0 0.0 0.0");
                //STLOutputFile->Add("    facet");
                STLOutputFile->Add("      outer loop");
                STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));
                STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[1].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[1].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[1].z,ffFixed,10,10));
                STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[2].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[2].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[2].z,ffFixed,10,10));
                STLOutputFile->Add("      endloop");
                STLOutputFile->Add("    endfacet");

                //high X end
                radius = NominalWorkSurface - Depths[Depths.Length - 1][i];
                angle = ((double)(i)) * angleperpixel;
                FacetCorners[0].x = XOffset + lengthperpixel * (double)(Depths.Length-1);
                FacetCorners[0].y = radius * cos(angle);
                FacetCorners[0].z = radius * sin(angle);
                //corner 2 of surface
                radius = NominalWorkSurface - Depths[Depths.Length - 1][i + 1];
                angle = ((double)(i + 1)) * angleperpixel;
                FacetCorners[1].x = XOffset + lengthperpixel * (double)(Depths.Length-1);
                FacetCorners[1].y = radius * cos(angle);
                FacetCorners[1].z = radius * sin(angle);
                //corner 3 of surface
                FacetCorners[2].x = XOffset + lengthperpixel * (double)(Depths.Length-1);
                FacetCorners[2].y = 0.0;
                FacetCorners[2].z = 0.0;
                //Now to create normal & facet
                STLOutputFile->Add("    facet normal 1 0 0");
                //STLOutputFile->Add("    facet");
                STLOutputFile->Add("      outer loop");
                STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));
                STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[1].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[1].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[1].z,ffFixed,10,10));
                STLOutputFile->Add("        vertex " + FloatToStrF(FacetCorners[2].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[2].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[2].z,ffFixed,10,10));
                STLOutputFile->Add("      endloop");
                STLOutputFile->Add("    endfacet");

                }


            //End
            STLOutputFile->Add("endsolid");

            Label3->Caption = "Writing STL file";
            Refresh();

            STLOutputFile->SaveToFile(ChangeFileExt(Edit2->Text,".stl"));
            delete STLOutputFile;

            Refresh();
            }  //end of try  statement

        __finally {
            SetInterface(true);
            //get rid of depth array
            for (int i = 0; i < Depths.Length; i++) {
                Depths[i].Length = 0;
                }
            Depths.Length = 0;
            //end
            Label3->Caption = "Complete";
            Refresh();
            }

        } else {
        if (FileExists(Edit1->Text)) {
            Application->MessageBox("An existing STL file will get overwritten.\n\nPlease delete file or change converted file name","Warning", MB_OK | MB_ICONERROR);
            } else {
            Application->MessageBox("The input file does not exist.\n\nPlease check","Warning", MB_OK | MB_ICONERROR);
            }
        Label3->Caption = "Conversion Aborted";
        }


}
//---------------------------------------------------------------------------

nodalposition __fastcall TDepthCutForm::UnitVector(nodalposition vector1)
{
    //unitises the vector
    double length = vector1.x * vector1.x + vector1.y * vector1.y + vector1.z * vector1.z;
    length = sqrt(length);

    if (length) {
        vector1.x = vector1.x / length;
        vector1.y = vector1.y / length;
        vector1.z = vector1.z / length;
        } else {
        //there's a problem if we get here.
        vector1.x = 1.0;
        vector1.y = 0.0;
        vector1.z = 0.0;
        }

    return vector1;
}
//---------------------------------------------------------------------------

nodalposition __fastcall TDepthCutForm::VectorProduct(nodalposition vector1, nodalposition vector2)
{
    //returns the vector product of two 3D vectors
    nodalposition value;

    value.x = vector1.y * vector2.z - vector1.z * vector2.y;
    value.y = vector1.z * vector2.x - vector1.x * vector2.z;
    value.z = vector1.x * vector2.y - vector1.y * vector2.x;

    return value;

}
//---------------------------------------------------------------------------

nodalposition __fastcall TDepthCutForm::SubtractVector(nodalposition Node1, nodalposition Node2)
{
    nodalposition total;

    total.x = Node1.x - Node2.x;
    total.y = Node1.y - Node2.y;
    total.z = Node1.z - Node2.z;

    return total;
}
//---------------------------------------------------------------------------

void __fastcall TDepthCutForm::CreateOBJButtonClick(TObject *Sender)
{
    //similarly to the DXF file creation above, this creates an OBJ file
    if (!Edit1->Text.Length() || !Edit2->Text.Length()) {
        Application->MessageBox("Need two file names\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (Edit1->Text.LowerCase() == ChangeFileExt(Edit2->Text.LowerCase(),".obj")) {
        Application->MessageBox("Both file names the same\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (!Edit4->Text.Length()) {
        Application->MessageBox("Please enter a maximum value for\nthe angle subtended by the image\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit5->Text,0.0) == 0.0) {
        Application->MessageBox("The Image has zero length along the X Axis.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit6->Text,0.0) == 0.0) {
        Application->MessageBox("The nominal work surface is at the rotary Axis centre.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit7->Text,0.0) == 0.0) {
        Application->MessageBox("The depth of cut is zero, so no machining will occur.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (FileExists(Edit1->Text) && !FileExists(ChangeFileExt(Edit2->Text,".obj"))) {

        //Create Depth Array
        DynamicArray< DynamicArray < double > > Depths;
        Depths.Length = Picture->Width;
        for (int i = 0; i < Depths.Length; i++) {
            Depths[i].Length = Picture->Height + 1;
            }
        DynamicArray< DynamicArray < int > > NodeNumbers;
        NodeNumbers.Length = Picture->Width;
        for (int i = 0; i < NodeNumbers.Length; i++) {
            NodeNumbers[i].Length = Picture->Height + 1;
            }

        try {
            SetInterface(false);

            //obtain values from input data
            double XOffset = ToDoubleDef(Edit3->Text,0.0);
            double TotalAngle = ToDoubleDef(Edit4->Text,360.0);
            //angle per pixel in degrees
            double angleperpixel = (M_PI * TotalAngle)/(180.0 * (double) Picture->Height);
            double TotalLength = ToDoubleDef(Edit5->Text,100.0);
            double lengthperpixel = TotalLength/((double) (Picture->Width - 1));
            double NominalWorkSurface = ToDoubleDef(Edit6->Text,7.5);
            double DepthofCut = ToDoubleDef(Edit7->Text,1.0);
            double depthfactor;
            double angle;
            double radius;
            DynamicArray <nodalposition> FacetCorners;
            FacetCorners.Length = 1;


            //go through pixels & find the maximum brightness of the pixels in the picture
            int maxbrightness = 0;
            int localbrightness;
            TColor pixelcolour;
            int red;
            int green;
            int blue;

            //first loop to find out maxbrightness values
            Label3->Caption = "Checking max brightness values";
            Refresh();

            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    if (localbrightness > maxbrightness) maxbrightness = localbrightness;
                    }
                }
            //right, max brightness now holds the peak brightness
            depthfactor = DepthofCut/((double)maxbrightness);

            //Assign depths
            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    Depths[i][j] = ((double) localbrightness) * depthfactor;
                    }
                }
            //now all the depths are assigned, except the last row, which is a copy of the first
            //this simplifies the closure process.
            for (int i = 0; i < Picture->Width; i++) {
                Depths[i][Depths[i].Length - 1] = Depths[i][0];
                }


            //right, all the individual depths are stored in the array, now to write an OBJ file
            Label3->Caption = "Creating OBJ file";
            Refresh();
            TStringList *OBJOutputFile = new TStringList();
            //Add Header
            OBJOutputFile->Add("# OBJ File Produced by the Chestnut Pens Depth Mill Program");

            //Create All Vertices
            int counter = 1;
            for (int i = 0; i < Depths.Length; i++) {
                for (int j = 0; j < Depths[i].Length; j++) {
                    radius = NominalWorkSurface - Depths[i][j];
                    angle = ((double)(j)) * angleperpixel;
                    FacetCorners[0].x = XOffset + lengthperpixel * (double)(i);
                    FacetCorners[0].y = radius * cos(angle);
                    FacetCorners[0].z = radius * sin(angle);
                    OBJOutputFile->Add("v " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));
                    NodeNumbers[i][j] = counter;
                    counter++;
                    }
                }
            counter--;
            //counter now holds the max number of vertices in the cylinder

            //now the extra vertices on the axes at either end
            FacetCorners[0].x = XOffset;
            FacetCorners[0].y = 0.0;
            FacetCorners[0].z = 0.0;
            OBJOutputFile->Add("v " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));
            FacetCorners[0].x = XOffset + lengthperpixel * (double)(Depths.Length-1);
            OBJOutputFile->Add("v " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));


            //Create All Texture Vertices
            for (int i = 0; i < Depths.Length; i++) {
                for (int j = 0; j < Depths[i].Length; j++) {
                    FacetCorners[0].x = ((double)i)/((double) Depths.Length-1);
                    FacetCorners[0].y = 1.0 - ((double)j)/((double) Depths[i].Length-1);
                    OBJOutputFile->Add("vt " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10));
                    }
                }

            //Create All Vertex Normals
            //Taken as radial for the cylinder
            for (int i = 0; i < Depths[0].Length; i++) {
                angle = ((double)(i)) * angleperpixel;
                FacetCorners[0].x = 0.0;
                FacetCorners[0].y = cos(angle);
                FacetCorners[0].z = sin(angle);
                OBJOutputFile->Add("vn " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));
                }
            //now the ends of the cylinder
            FacetCorners[0].x = -1.0;
            FacetCorners[0].y = 0.0;
            FacetCorners[0].z = 0.0;
            OBJOutputFile->Add("vn " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));
            FacetCorners[0].x = 1.0;
            OBJOutputFile->Add("vn " + FloatToStrF(FacetCorners[0].x,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].y,ffFixed,10,10) + " " + FloatToStrF(FacetCorners[0].z,ffFixed,10,10));

            //write the facets
            AnsiString temp;
            OBJOutputFile->Add("g cylinder");
            OBJOutputFile->Add("usemtl cylindermaterial");
            for (int i = 0; i < Depths.Length - 1; i++) {
                for (int j = 0; j < Depths[i].Length - 1; j++) {
                    temp = "f ";
                    temp = temp + AnsiString(NodeNumbers[i][j]) + "/" + AnsiString(NodeNumbers[i][j]) + "/" + AnsiString(j + 1) + " ";
                    temp = temp + AnsiString(NodeNumbers[i][j + 1]) + "/" + AnsiString(NodeNumbers[i][j+1]) + "/" + AnsiString(j + 2) + " ";
                    temp = temp + AnsiString(NodeNumbers[i+1][j + 1]) + "/" + AnsiString(NodeNumbers[i+1][j+1]) + "/" + AnsiString(j + 2) + " ";
                    temp = temp + AnsiString(NodeNumbers[i+1][j]) + "/" + AnsiString(NodeNumbers[i+1][j]) + "/" + AnsiString(j + 1);
                    OBJOutputFile->Add(temp);
                    }
                }

            OBJOutputFile->Add("g ends");
            OBJOutputFile->Add("usemtl endmaterial");
            int totalnormals = Depths[0].Length;
            for (int j = 0; j < Depths[0].Length - 1; j++) {
                temp = "f ";
                temp = temp + AnsiString(NodeNumbers[0][j + 1]) + "//" + AnsiString(totalnormals + 1) + " ";
                temp = temp + AnsiString(NodeNumbers[0][j]) + "//" + AnsiString(totalnormals + 1) + " ";
                temp = temp + AnsiString(counter + 1) + "//" + AnsiString(totalnormals + 1);
                OBJOutputFile->Add(temp);
                temp = "f ";
                temp = temp + AnsiString(NodeNumbers[NodeNumbers.Length - 1][j + 1]) + "//" + AnsiString(totalnormals + 2) + " ";
                temp = temp + AnsiString(NodeNumbers[NodeNumbers.Length - 1][j]) + "//" + AnsiString(totalnormals + 2) + " ";
                temp = temp + AnsiString(counter + 2) + "//" + AnsiString(totalnormals + 2);
                OBJOutputFile->Add(temp);
                }




            //End
            OBJOutputFile->Add("# end");

            Label3->Caption = "Writing OBJ file";
            Refresh();

            OBJOutputFile->SaveToFile(ChangeFileExt(Edit2->Text,".obj"));
            delete OBJOutputFile;
            FacetCorners.Length = 0;

            Refresh();
            }  //end of try  statement

        __finally {
            SetInterface(true);
            //get rid of depth array
            for (int i = 0; i < Depths.Length; i++) {
                Depths[i].Length = 0;
                NodeNumbers[i].Length = 0;
                }
            Depths.Length = 0;
            NodeNumbers.Length = 0;
            //end
            Label3->Caption = "Complete";
            Refresh();
            }

        } else {
        if (FileExists(Edit1->Text)) {
            Application->MessageBox("An existing OBJ file will get overwritten.\n\nPlease delete file or change converted file name","Warning", MB_OK | MB_ICONERROR);
            } else {
            Application->MessageBox("The input file does not exist.\n\nPlease check","Warning", MB_OK | MB_ICONERROR);
            }
        Label3->Caption = "Conversion Aborted";
        }



}
//---------------------------------------------------------------------------

void __fastcall TDepthCutForm::Edit6Change(TObject *Sender)
{
    if (ToDoubleDef(Edit6->Text,7.5) > ToDoubleDef(Edit9->Text,0.0)) {
        Edit9->Text = AnsiString( ToDoubleDef(Edit6->Text,7.5) + 1.0);
        }
}
//---------------------------------------------------------------------------

void __fastcall TDepthCutForm::CreateNCButtonClick(TObject *Sender)
{
    //similarly to the DXF file creation above, this creates an NC Toolpath file
    if (!Edit1->Text.Length() || !Edit2->Text.Length()) {
        Application->MessageBox("Need two file names\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (Edit1->Text.LowerCase() == ChangeFileExt(Edit2->Text.LowerCase(),".nc")) {
        Application->MessageBox("Both file names the same\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (!Edit4->Text.Length()) {
        Application->MessageBox("Please enter a maximum value for\nthe angle subtended by the image\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit5->Text,0.0) == 0.0) {
        Application->MessageBox("The Image has zero length along the X Axis.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit6->Text,0.0) == 0.0) {
        Application->MessageBox("The nominal work surface is at the rotary Axis centre.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (ToDoubleDef(Edit7->Text,0.0) == 0.0) {
        Application->MessageBox("The depth of cut is zero, so no machining will occur.\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (FileExists(Edit1->Text) && !FileExists(ChangeFileExt(Edit2->Text,".nc"))) {

        //Create Depth Array
        DynamicArray< DynamicArray < double > > Depths;
        Depths.Length = Picture->Width;
        for (int i = 0; i < Depths.Length; i++) {
            Depths[i].Length = Picture->Height;
            }

        try {
            SetInterface(false);

            //obtain values from input data
            double XOffset = ToDoubleDef(Edit3->Text,0.0);
            double TotalAngle = ToDoubleDef(Edit4->Text,360.0);
            //angle per pixel in degrees
            double angleperpixel = (TotalAngle)/((double) Picture->Height);
            double TotalLength = ToDoubleDef(Edit5->Text,100.0);
            double lengthperpixel = TotalLength/((double) (Picture->Width - 1));
            double NominalWorkSurface = ToDoubleDef(Edit6->Text,7.5);
            double DepthofCut = ToDoubleDef(Edit7->Text,1.0);
            double Feedrate = ToDoubleDef(Edit8->Text,200.0);
            double ClearanceZ = ToDoubleDef(Edit9->Text,10.0);
            double depthfactor;
            double angle;
            double radius;
            //DynamicArray <nodalposition> FacetCorners;
            //FacetCorners.Length = 1;
            AnsiString GCodeHeader = Edit10->Text;
            AnsiString GCodeFooter = Edit11->Text;


            //go through pixels & find the maximum brightness of the pixels in the picture
            int maxbrightness = 0;
            int localbrightness;
            TColor pixelcolour;
            int red;
            int green;
            int blue;

            //first loop to find out maxbrightness values
            Label3->Caption = "Checking max brightness values";
            Refresh();

            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    if (localbrightness > maxbrightness) maxbrightness = localbrightness;
                    }
                }
            //right, max brightness now holds the peak brightness
            depthfactor = DepthofCut/((double)maxbrightness);

            //Assign depths
            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    Depths[i][j] = ((double) localbrightness) * depthfactor;
                    }
                }
            //now all the depths are assigned,


            //right, all the individual depths are stored in the array, now to write an NC file
            Label3->Caption = "Creating Toolpath file";
            Refresh();
            TStringList *NCOutputFile = new TStringList();
            //Add Header
            NCOutputFile->Add("( Toolpath File Produced by the Chestnut Pens Depth Mill Program )");
            NCOutputFile->Add("( Left Edge of Picture at X = " + AnsiString(XOffset) + "mm )");
            NCOutputFile->Add("( Picture is wrapped around " + AnsiString(TotalAngle) + "degrees )");
            NCOutputFile->Add("( Z Value of worksurface " + AnsiString(NominalWorkSurface) + "mm )");
            NCOutputFile->Add("( Feedrate = " + AnsiString(Feedrate) + "mm/min )");
            NCOutputFile->Add("( Max Depth of Cut = " + AnsiString(DepthofCut) + "mm )");
            NCOutputFile->Add("( Clearance Plane Z = " + AnsiString(ClearanceZ) + "mm )");
            NCOutputFile->Add("( No tool diameter is assumed, use whatever ball nose cutter is suitable )");
            NCOutputFile->Add("( ---------------------------------------------------------------------- )");
            NCOutputFile->Add("( Header )");
            NCOutputFile->Add(GCodeHeader);
            NCOutputFile->Add("(XY Plane, Metric, Absolute distance mode, mm/min feedrate)");
            NCOutputFile->Add("G17 G21 G90 G90.1 G94");
            NCOutputFile->Add("(Tool 0, Spindle 1000 rpm)");
            NCOutputFile->Add("T0 S1000");
            NCOutputFile->Add("(Move to retract plane)");
            NCOutputFile->Add("G0 Z" + DP(4,ClearanceZ));
            NCOutputFile->Add("(Move to start position)");
            NCOutputFile->Add("G0 X0 Y0 A0");
            NCOutputFile->Add("(Half speed feed for first line as it's being plunged)");
            NCOutputFile->Add("F" + DP(2,Feedrate/2.0));

            //Create All Vertices
            int index = 0;
            double X;
            bool firstloop = true;
            while (index < Depths[0].Length) {
                for (int j = 0; j < Depths.Length; j++) {
                    radius = NominalWorkSurface - Depths[j][index];
                    X = XOffset + ((double)j) * lengthperpixel;
                    angle = ((double)(index)) * angleperpixel;
                    if (j == 0 || j == Depths.Length - 1) {
                        NCOutputFile->Add("G1 X" + DP(4,X) + " Z" + DP(4,radius) + " A" + DP(4,angle));
                        } else {
                        NCOutputFile->Add("G1 X" + DP(4,X) + " Z" + DP(4,radius));
                        }
                    }
                index++;
                if (firstloop) {
                    //full feedrate at end of first plunge cut
                    firstloop = false;
                    NCOutputFile->Add("F" + DP(2,Feedrate));
                    }
                if (index < Depths[0].Length) {
                    for (int j = (Depths.Length - 1); j > -1; j--) {
                        radius = NominalWorkSurface - Depths[j][index];
                        X = XOffset + ((double)j) * lengthperpixel;
                        angle = ((double)(index)) * angleperpixel;
                        if (j == 0 || j == Depths.Length - 1) {
                            NCOutputFile->Add("G1 X" + DP(4,X) + " Z" + DP(4,radius) + " A" + DP(4,angle));
                            } else {
                            NCOutputFile->Add("G1 X" + DP(4,X) + " Z" + DP(4,radius));
                            }
                        }
                    index++;
                    }
                }
            //Closure & Footer
            NCOutputFile->Add("( End )");
            NCOutputFile->Add("G0 Z" + DP(4,ClearanceZ));
            NCOutputFile->Add("G0 A0 X0");
            NCOutputFile->Add(GCodeFooter);

            Label3->Caption = "Writing NC file";
            Refresh();

            NCOutputFile->SaveToFile(ChangeFileExt(Edit2->Text,".nc"));
            delete NCOutputFile;
            //FacetCorners.Length = 0;

            Refresh();
            }  //end of try  statement

        __finally {
            SetInterface(true);
            //get rid of depth array
            for (int i = 0; i < Depths.Length; i++) {
                Depths[i].Length = 0;
                }
            Depths.Length = 0;
            //end
            Label3->Caption = "Complete";
            Refresh();
            }

        } else {
        if (FileExists(Edit1->Text)) {
            Application->MessageBox("An existing NC file will get overwritten.\n\nPlease delete file or change converted file name","Warning", MB_OK | MB_ICONERROR);
            } else {
            Application->MessageBox("The input file does not exist.\n\nPlease check","Warning", MB_OK | MB_ICONERROR);
            }
        Label3->Caption = "Conversion Aborted";
        }




}
//---------------------------------------------------------------------------

void __fastcall TDepthCutForm::SetInterface(bool Setting)
{
            CreateDXFButton->Enabled = Setting;
            CreateSTLButton->Enabled = Setting;
            CreateOBJButton->Enabled = Setting;
            CreateNCButton->Enabled = Setting;
            OriginalBMPButton->Enabled = Setting;
            NewDXFButton->Enabled = Setting;
            HelpButton->Enabled = Setting;
            CloseButton->Enabled = Setting;
            Edit1->Enabled = Setting;
            Edit2->Enabled = Setting;
            Edit3->Enabled = Setting;
            Edit4->Enabled = Setting;
            Edit5->Enabled = Setting;
            Edit6->Enabled = Setting;
            Edit7->Enabled = Setting;
            Edit8->Enabled = Setting;
            Edit9->Enabled = Setting;
            Edit10->Enabled = Setting;
            Edit11->Enabled = Setting;
            Refresh();

}
//---------------------------------------------------------------------------

void __fastcall TDepthCutForm::SetButtons(bool Setting)
{
            CreateDXFButton->Enabled = Setting;
            CreateSTLButton->Enabled = Setting;
            CreateOBJButton->Enabled = Setting;
            CreateNCButton->Enabled = Setting;
            Refresh();

}
//---------------------------------------------------------------------------


void __fastcall TDepthCutForm::Edit1Change(TObject *Sender)
{
    //This is what happens if Edit 1 Changes without the browse button being used
    if (FileExists(Edit1->Text) && ExtractFileExt(Edit1->Text).LowerCase() == ".bmp") {
        Edit2->Text = ChangeFileExt(Edit1->Text, " Depth Mill.dxf");
        SaveDialog1->InitialDir = ExtractFilePath(Edit1->Text);
        try {
            Picture->LoadFromFile(Edit1->Text);
            Label3->Caption = "Picture loaded and is = " + AnsiString(Picture->Width) + "W x " + AnsiString(Picture->Height) + "H Pixels";
            ConversionPossible = true;
            SetButtons(true);
            }
        catch (...) {
            Label3->Caption = "Picture did not load";
            ConversionPossible = false;
            SetButtons(false);
            }
        } else {
        SetButtons(false);
        }
}
//---------------------------------------------------------------------------

